#include "rar.hpp"
#include "global.hpp"
#include "savepos.hpp"
#include "filefn.hpp"
#include "array.hpp"
#include "unicode.hpp"

static File *CreatedFiles[256];
static int RemoveCreatedActive=0;

File::File()
{
  hFile=BAD_HANDLE;
  *FileName=0;
  *FileNameW=0;
  SkipClose=false;
  OpenShared=false;
  CloseCount=0;
}


File::~File()
{
  if (hFile!=BAD_HANDLE && !SkipClose)
      Close();
}


void File::operator = (File &SrcFile)
{
  hFile=SrcFile.hFile;
  strcpy(FileName,SrcFile.FileName);
  SrcFile.SkipClose=true;
}


bool File::Open(const char *Name,const wchar *NameW,bool OpenShared,bool Update)
{
  FileHandle hNewFile;
  if (File::OpenShared)
    OpenShared=true;
  hNewFile = sceIoOpen(Name, PSP_O_RDONLY, 0444);
  pos = 0;
  SkipClose=false;
  bool Success=hNewFile!=BAD_HANDLE;
  if (Success)
  {
    hFile=hNewFile;
    if (NameW!=NULL)
      strcpyw(FileNameW,NameW);
    else
      *FileNameW=0;
    if (Name!=NULL)
      strcpy(FileName,Name);
    else
      WideToChar(NameW,FileName);
    AddFileToList(hFile);
  }else{
    SkipClose=true;
  }
  return(Success);
}

void File::AddFileToList(FileHandle hFile)
{
  if (hFile!=BAD_HANDLE)
    for (int I=0;I<sizeof(CreatedFiles)/sizeof(CreatedFiles[0]);I++)
      if (CreatedFiles[I]==NULL)
      {
        CreatedFiles[I]=this;
        break;
      }
}

bool File::Close()
{
  bool Success=true;
    if (hFile!=BAD_HANDLE)
    {
      if (!SkipClose)
      {
	sceIoClose(hFile);
        if (Success || !RemoveCreatedActive)
          for (int I=0;I<sizeof(CreatedFiles)/sizeof(CreatedFiles[0]);I++)
            if (CreatedFiles[I]==this)
            {
              CreatedFiles[I]=NULL;
              break;
            }
      }
      hFile=BAD_HANDLE;
      if (!Success)
        ErrHandler.CloseError(FileName);
    }
  CloseCount++;
  return(Success);
}

int File::Read(void *Data,int Size)
{
  Int64 FilePos;
  int ReadSize;
  while (true)
  {
    ReadSize=DirectRead(Data,Size);
    if (ReadSize==-1)
    {
      ErrHandler.ReadError(FileName);
    }
    break;
  }
  return(ReadSize);
}


int File::DirectRead(void *Data,int Size)
{
  pos += Size;
  return sceIoRead(hFile, Data, Size);
}


void File::Seek(Int64 Offset,int Method)
{
  if (!RawSeek(Offset,Method))
    ErrHandler.SeekError(FileName);
}


bool File::RawSeek(Int64 Offset,int Method)
{
  if (hFile==BAD_HANDLE)
    return(true);
  if (!is64plus(Offset) && Method!=SEEK_SET)
  {
    Offset=(Method==SEEK_CUR ? Tell():FileLength())+Offset;
    Method=SEEK_SET;
  }
  sceIoLseek(hFile, int64to32(Offset), Method);
  pos = int64to32(Offset);
  return(true);
}


Int64 File::Tell()
{
  return pos;
}

byte File::GetByte()
{
  byte Byte=0;
  Read(&Byte,1);
  return(Byte);
}

Int64 File::FileLength()
{
  SaveFilePos SavePos(*this);
  Seek(0,SEEK_END);
  return(Tell());
}
